/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 * 
 * Based on devzone guide...
 * https://devzone.nordicsemi.com/guides/nrf-connect-sdk-guides/b/software/posts/ncs-dfu
 * 
 * 
 * When you are working with child-images, you may want to configure the child-image with 
 * your specific configuration, the way this is done is by adding a child-image folder in
 * your application folder (child_image\mcuboot\boards\nrf52833dk_nrf52833.conf). There 
 * are several ways this could and can be done, as described here:
 * 
 * generic:
 * https://developer.nordicsemi.com/nRF_Connect_SDK/doc/latest/nrf/app_dev/multi_image/index.html 
 * 
 * specific: 
 * https://developer.nordicsemi.com/nRF_Connect_SDK/doc/latest/nrf/app_dev/multi_image/index.html#image-specific-variables 
 * 
 */

#include <zephyr/kernel.h>
#include <dk_buttons_and_leds.h>
#include <zephyr/drivers/gpio.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>

// Additions for DFU
#include <zephyr/mgmt/mcumgr/smp_bt.h>
#include "os_mgmt/os_mgmt.h"
#include "img_mgmt/img_mgmt.h"


#define DEVICE_NAME             CONFIG_BT_DEVICE_NAME		// prj.conf
#define DEVICE_NAME_LEN         (sizeof(DEVICE_NAME) - 1)

#define CON_STATUS_LED          1	// DK_LED2

// Advertisting data
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};

// Scan response data.  This is the Bluetooth SIG SMP UUID 
static const struct bt_data sd[] = {
//	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BT_UUID_LBS_VAL),
	BT_DATA_BYTES(BT_DATA_UUID128_ALL,
		      0x84, 0xaa, 0x60, 0x74, 0x52, 0x8a, 0x8b, 0x86,
		      0xd3, 0x4c, 0xb7, 0x1d, 0x1d, 0xdc, 0x53, 0x8d),
};

static void connected(struct bt_conn *conn, uint8_t err)
{
	if (err) {
		printk("Connection failed (err %u)\n", err);
		return;
	}

	printk("Connected\n");

	dk_set_led_on(CON_STATUS_LED);
}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	printk("Disconnected (reason %u)\n", reason);

	dk_set_led_off(CON_STATUS_LED);
}


BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected        = connected,
	.disconnected     = disconnected,
};


#define SW0_NODE	DT_ALIAS(sw0)
static const struct gpio_dt_spec button = GPIO_DT_SPEC_GET(SW0_NODE, gpios);

#define LED1_NODE DT_ALIAS(led1)
const struct gpio_dt_spec led_1 = GPIO_DT_SPEC_GET(LED1_NODE, gpios);


void main(void)
{
	int ret;
	bool val;

	if (!device_is_ready(button.port)) {
		return;
	}

	ret = gpio_pin_configure_dt(&button, GPIO_INPUT);
	if (ret < 0) {
		return;
	}

	// LED
	if (!device_is_ready(led_1.port)) {
		printk("DK LED1 not ready\n");
		return;
	}

	ret = gpio_pin_configure_dt(&led_1, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		printk("Cannot configure DK LED1 (err: %d)\n", ret);
		return;
	}


printk("Hello World 2! %s\n", CONFIG_BOARD);


    // val = gpio_pin_get_dt(&button);
	// if(val){
		// Start the bootloader
//		printk("Release the button\n");
		k_sleep(K_MSEC(2000));

		printk("Starting Bootloader\n");
		
		os_mgmt_register_group();
		img_mgmt_register_group();
		smp_bt_register();	

		ret = bt_enable(NULL);
		if (ret) {
			printk("Bluetooth init failed (err %d)\n", ret);
			return;
		}

		printk("Bluetooth initialized\n");

		ret = bt_le_adv_start(BT_LE_ADV_CONN, ad, ARRAY_SIZE(ad),
					sd, ARRAY_SIZE(sd));
		if (ret) {
			printk("Advertising failed to start (err %d)\n", ret);
			return;
		}else{
			printk("Advertising successfully started\n");
		}

		for (;;) {
			gpio_pin_toggle_dt(&led_1);
			k_sleep(K_MSEC(500));
		}


	// }else{
	// 	printk("Hello World 1! %s\n", CONFIG_BOARD);
	// }
	



}
